import { __spread, __decorate } from 'tslib';
import { Input, HostBinding, HostListener, Directive, NgModule } from '@angular/core';
import { DomSanitizer } from '@angular/platform-browser';

var isJsons = function (array) {
    return Array.isArray(array) &&
        array.every(function (row) { return typeof row === 'object' && !(row instanceof Array); });
};
var isArrays = function (array) {
    return Array.isArray(array) && array.every(function (row) { return Array.isArray(row); });
};
function jsonsHeaders(array) {
    return Array.from(new Set(array.map(function (item) { return Object.keys(item); }).reduce(function (a, b) { return __spread(a, b); }, [])));
}
function jsons2arrays(jsons, headers) {
    headers = headers || jsonsHeaders(jsons);
    // allow headers to have custom labels, defaulting to having the header data key be the label
    var headerLabels = headers;
    var headerKeys = headers;
    if (isJsons(headers)) {
        headerLabels = headers.map(function (header) { return header.label; });
        headerKeys = headers.map(function (header) { return header.key; });
    }
    var data = jsons.map(function (object) {
        return headerKeys.map(function (header) { return (header in object ? object[header] : ''); });
    });
    return __spread([headerLabels], data);
}
var elementOrEmpty = function (element) {
    return element || element === 0 ? element : '';
};
function joiner(data, delimiter) {
    if (delimiter === void 0) { delimiter = ','; }
    return data
        .map(function (row, index) {
        return row.map(function (element) { return '"' + elementOrEmpty(element) + '"'; }).join(delimiter);
    })
        .join("\n");
}
function arrays2csv(data, headers, delimiter) {
    return joiner(headers ? __spread([headers], data) : data, delimiter);
}
function jsons2csv(data, headers, delimiter) {
    return joiner(jsons2arrays(data, headers), delimiter);
}
function string2csv(data, headers, delimiter) {
    return headers ? headers.join(delimiter) + "\n" + data : data;
}
function toCSV(data, headers, delimiter) {
    if (isJsons(data)) {
        return jsons2csv(data, headers, delimiter);
    }
    if (isArrays(data)) {
        return arrays2csv(data, headers, delimiter);
    }
    if (typeof data === 'string') {
        return string2csv(data, headers, delimiter);
    }
    throw new TypeError("Data should be a \"String\", \"Array of arrays\" OR \"Array of objects\" ");
}
function blob(data, uFEFF, headers, delimiter) {
    if (uFEFF === void 0) { uFEFF = true; }
    var csv = toCSV(data, headers, delimiter);
    return new Blob([uFEFF ? '\uFEFF' : '', csv], { type: 'text/csv' });
}
function buildURI(data, uFEFF, headers, delimiter) {
    if (uFEFF === void 0) { uFEFF = true; }
    return URL.createObjectURL(blob(data, uFEFF, headers, delimiter));
}

var CsvDirective = /** @class */ (function () {
    function CsvDirective(sanitizer) {
        this.sanitizer = sanitizer;
        /** the body of the csv */
        this.data = [];
        /** Set the seperator between values */
        this.delimiter = ',';
        /** adds a Byte order mark to setup the csv as UTF-8  */
        this.uFEFF = true;
        /** filename */
        this.download = 'data.csv';
        this.target = this.isIEBrowser() ? '' : '_blank';
    }
    Object.defineProperty(CsvDirective.prototype, "filename", {
        /** Set the filename of the csv. Default is `data.csv` */
        set: function (a) {
            this.download = a;
        },
        enumerable: true,
        configurable: true
    });
    CsvDirective.prototype.onClick = function () {
        // IE handling
        if (this.isIEBrowser()) {
            var file = blob(this.data, this.uFEFF, this.headers, this.delimiter);
            window.navigator.msSaveBlob(file, this.download);
        }
    };
    CsvDirective.prototype.isIEBrowser = function () {
        return !!window.navigator.msSaveOrOpenBlob;
    };
    CsvDirective.prototype.ngOnChanges = function () {
        this.href = this.sanitizer.bypassSecurityTrustResourceUrl(buildURI(this.data, this.uFEFF, this.headers, this.delimiter));
    };
    CsvDirective.ctorParameters = function () { return [
        { type: DomSanitizer }
    ]; };
    __decorate([
        Input()
    ], CsvDirective.prototype, "data", void 0);
    __decorate([
        Input()
    ], CsvDirective.prototype, "headers", void 0);
    __decorate([
        Input()
    ], CsvDirective.prototype, "delimiter", void 0);
    __decorate([
        Input()
    ], CsvDirective.prototype, "filename", null);
    __decorate([
        Input()
    ], CsvDirective.prototype, "uFEFF", void 0);
    __decorate([
        HostBinding()
    ], CsvDirective.prototype, "href", void 0);
    __decorate([
        HostBinding()
    ], CsvDirective.prototype, "download", void 0);
    __decorate([
        Input(), HostBinding()
    ], CsvDirective.prototype, "target", void 0);
    __decorate([
        HostListener('click')
    ], CsvDirective.prototype, "onClick", null);
    CsvDirective = __decorate([
        Directive({ selector: '[csvLink]' })
    ], CsvDirective);
    return CsvDirective;
}());

var CsvModule = /** @class */ (function () {
    function CsvModule() {
    }
    CsvModule = __decorate([
        NgModule({
            declarations: [CsvDirective],
            exports: [CsvDirective],
        })
    ], CsvModule);
    return CsvModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { CsvDirective, CsvModule, arrays2csv, blob, buildURI, elementOrEmpty, isArrays, isJsons, joiner, jsons2arrays, jsons2csv, jsonsHeaders, string2csv, toCSV };
//# sourceMappingURL=ctrl-ngx-csv.js.map
